@@ -6,16 +6,20 @@ package org.openstreetmap.josm.data.gpx;
 import java.text.ParsePosition;
 import java.text.SimpleDateFormat;
 import java.util.Date;
+import java.awt.Color;
 
 import org.openstreetmap.josm.Main;
 import org.openstreetmap.josm.data.coor.EastNorth;
 import org.openstreetmap.josm.data.coor.LatLon;
 
-public class WayPoint extends WithAttributes {
+public class WayPoint extends WithAttributes implements Comparable{
 	
 	public final LatLon latlon;
 	public final EastNorth eastNorth;
 	public double time;
+	public Color speedLineColor;
+	public boolean drawLine;
+	public int dir;
 
 	public WayPoint(LatLon ll) {
 		latlon = ll; 
@@ -28,8 +32,7 @@ public class WayPoint extends WithAttributes {
 	}
 	
 	/**
-	 * convert the time stamp of ther waypoint into seconds from the epoch
-	 * @return seconds
+	 * Convert the time stamp of the waypoint into seconds from the epoch
 	 */
 	public void setTime () {
 		if (! attr.containsKey("time")) {
@@ -45,4 +48,11 @@ public class WayPoint extends WithAttributes {
 		}
 	}
 
+    public int compareTo(Object other){
+        if(other instanceof WayPoint){
+            WayPoint w = (WayPoint)other;
+            return (int)time - (int)w.time;
+        }
+        return 0;
+    }
 }
