@@ -1,4 +1,5 @@
-// License: GPL. Copyright 2007 by Immanuel Scholz and others
+// License: GPL. See LICENSE file for details.
+
 package org.openstreetmap.josm.gui.dialogs;
 
 import static org.openstreetmap.josm.tools.I18n.tr;
@@ -28,15 +29,16 @@ import java.util.Map.Entry;
 
 import javax.swing.Box;
 import javax.swing.DefaultComboBoxModel;
+import javax.swing.DefaultListCellRenderer;
 import javax.swing.JButton;
 import javax.swing.JComboBox;
 import javax.swing.JDialog;
 import javax.swing.JLabel;
+import javax.swing.JList;
 import javax.swing.JOptionPane;
 import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.JTable;
-import javax.swing.JTextField;
 import javax.swing.ListSelectionModel;
 import javax.swing.table.DefaultTableCellRenderer;
 import javax.swing.table.DefaultTableModel;
@@ -44,6 +46,7 @@ import javax.swing.text.JTextComponent;
 
 import org.openstreetmap.josm.Main;
 import org.openstreetmap.josm.command.ChangePropertyCommand;
+import org.openstreetmap.josm.command.Command;
 import org.openstreetmap.josm.command.SequenceCommand;
 import org.openstreetmap.josm.data.SelectionChangedListener;
 import org.openstreetmap.josm.data.osm.DataSet;
@@ -112,12 +115,14 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 		}
 	}
 
+	private final Map<String, Map<String, Integer>> valueCount = new TreeMap<String, Map<String, Integer>>();
 	/**
 	 * Edit the value in the properties table row
 	 * @param row The row of the table from which the value is edited.
 	 */
 	void propertyEdit(int row) {
 		String key = propertyData.getValueAt(row, 0).toString();
+		objKey=key;
 		Collection<OsmPrimitive> sel = Main.ds.getSelected();
 		if (sel.isEmpty()) {
 			JOptionPane.showMessageDialog(Main.parent, tr("Please select the objects you want to change properties for."));
@@ -128,50 +133,79 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 		JPanel panel = new JPanel(new BorderLayout());
 		panel.add(new JLabel(msg), BorderLayout.NORTH);
 
+		final TreeMap<String, TreeSet<String>> allData = createAutoCompletionInfo(true);
+
 		JPanel p = new JPanel(new GridBagLayout());
 		panel.add(p, BorderLayout.CENTER);
-		
-		final JTextField keyField = new JTextField(key);
+
+		final AutoCompleteComboBox keys = new AutoCompleteComboBox();
+		keys.setPossibleItems(allData.keySet());
+		keys.setEditable(true);
+		keys.setSelectedItem(key);
+
 		p.add(new JLabel(tr("Key")), GBC.std());
 		p.add(Box.createHorizontalStrut(10), GBC.std());
-		p.add(keyField, GBC.eol().fill(GBC.HORIZONTAL));
-				
-		final JComboBox valueField = (JComboBox) propertyData.getValueAt(row, 1);
+		p.add(keys, GBC.eol().fill(GBC.HORIZONTAL));
+
+		final AutoCompleteComboBox values = new AutoCompleteComboBox();
+		values.setRenderer(new DefaultListCellRenderer() {
+			@Override public Component getListCellRendererComponent(JList list,  Object value, int index, boolean isSelected,  boolean cellHasFocus) {
+			    Component c = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
+			    if (c instanceof JLabel) {
+			    	String str = null;
+			    		str=(String) value;
+			    		if (valueCount.containsKey(objKey)){ 
+			    			Map<String, Integer> m=valueCount.get(objKey);
+			    			if (m.containsKey(str)) {
+			    				str+="("+m.get(str)+")";
+			    				c.setFont(c.getFont().deriveFont(Font.ITALIC+Font.BOLD));
+			    			}
+			    		}
+				    ((JLabel)c).setText(str);
+			    }
+			    return c;
+			}
+		});
+		values.setEditable(true);
+		updateListData(key, allData, values);
+		Map<String, Integer> m=(Map<String, Integer>)propertyData.getValueAt(row, 1);
+		final String selection= m.size()!=1?tr("<different>"):m.entrySet().iterator().next().getKey();
+		values.setSelectedItem(selection);
+		values.getEditor().setItem(selection);
 		p.add(new JLabel(tr("Value")), GBC.std());
 		p.add(Box.createHorizontalStrut(10), GBC.std());
-		p.add(valueField, GBC.eol().fill(GBC.HORIZONTAL));
+		p.add(values, GBC.eol().fill(GBC.HORIZONTAL));
+		addFocusAdapter(row, allData, keys, values);
 
-		final JOptionPane optionPane = new JOptionPane(panel, JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION){
+		final JOptionPane optionPane = new JOptionPane(panel, JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION) {
 			@Override public void selectInitialValue() {
-				valueField.requestFocusInWindow();
-				valueField.getEditor().selectAll();
+				values.requestFocusInWindow();
+				values.getEditor().selectAll();
 			}
 		};
 		final JDialog dlg = optionPane.createDialog(Main.parent, tr("Change values?"));
 
-		valueField.getEditor().addActionListener(new ActionListener() {
+		values.getEditor().addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				dlg.setVisible(false);
 				optionPane.setValue(JOptionPane.OK_OPTION);
 			}
 		});
 
-		String oldValue = valueField.getEditor().getItem().toString();
+		String oldValue = values.getEditor().getItem().toString();
 		dlg.setVisible(true);
 
 		Object answer = optionPane.getValue();
 		if (answer == null || answer == JOptionPane.UNINITIALIZED_VALUE ||
 				(answer instanceof Integer && (Integer)answer != JOptionPane.OK_OPTION)) {
-			valueField.getEditor().setItem(oldValue);
+			values.getEditor().setItem(oldValue);
 			return;
 		}
 
-		String value = valueField.getEditor().getItem().toString();
-		if (value.equals(tr("<different>")))
-			return;
+		String value = values.getEditor().getItem().toString();
 		if (value.equals(""))
 			value = null; // delete the key
-		String newkey = keyField.getText();
+		String newkey = keys.getEditor().getItem().toString();
 		if (newkey.equals("")) {
 			newkey = key;
 			value = null; // delete the key instead
@@ -179,15 +213,48 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 		if (key.equals(newkey) || value == null)
 			Main.main.undoRedo.add(new ChangePropertyCommand(sel, newkey, value));
 		else {
-			Main.main.undoRedo.add(new SequenceCommand(trn("Change properties of {0} object", "Change properties of {0} objects", sel.size(), sel.size()),
-					new ChangePropertyCommand(sel, key, null),
-					new ChangePropertyCommand(sel, newkey, value)));
+			Collection<Command> commands=new Vector<Command>();
+			commands.add(new ChangePropertyCommand(sel, key, null));
+			if (value.equals(tr("<different>"))) {
+				HashMap<String, Vector<OsmPrimitive>> map=new HashMap<String, Vector<OsmPrimitive>>();
+				for (OsmPrimitive osm: sel) {
+					String val=osm.keys.get(key);
+					if (map.containsKey(val)) {
+						map.get(val).add(osm);
+					} else {
+						Vector<OsmPrimitive> v = new Vector<OsmPrimitive>();
+						v.add(osm);
+						map.put(val, v);
+					}
+				}
+				for (Entry<String, Vector<OsmPrimitive>> e: map.entrySet()) {
+					commands.add(new ChangePropertyCommand(e.getValue(), newkey, e.getKey()));
+				}
+			} else {
+				commands.add(new ChangePropertyCommand(sel, newkey, value));
+			}
+			Main.main.undoRedo.add(new SequenceCommand(trn("Change properties of up to {0} object", "Change properties of up to {0} objects", sel.size(), sel.size()), commands));
 		}
 
-			selectionChanged(sel); // update whole table
+		selectionChanged(sel); // update whole table
 		Main.parent.repaint(); // repaint all - drawing could have been changed
 	}
 
+	/**
+     * @param key
+     * @param allData
+     * @param values
+     */
+    private void updateListData(String key, final TreeMap<String, TreeSet<String>> allData, final AutoCompleteComboBox values) {
+	    Collection<String> newItems;
+	    if (allData.containsKey(key)) {
+			newItems = allData.get(key);
+		} else {
+			newItems = Collections.emptyList();
+		}
+		values.setPossibleItems(newItems);
+    }
+
 	/**
 	 * This simply fires up an relation editor for the relation shown; everything else
 	 * is the editor's business.
@@ -211,23 +278,9 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 		}
 
 		JPanel p = new JPanel(new BorderLayout());
-		p.add(new JLabel("<html>"+trn("This will change {0} object.","This will change {0} objects.", sel.size(),sel.size())+"<br><br>"+tr("Please select a key")),
+		p.add(new JLabel("<html>"+trn("This will change up to {0} object.","This will change up to {0} objects.", sel.size(),sel.size())+"<br><br>"+tr("Please select a key")),
 				BorderLayout.NORTH);
-		final TreeMap<String,TreeSet<String>> allData = new TreeMap<String,TreeSet<String>>();
-		for (OsmPrimitive osm : Main.ds.allNonDeletedPrimitives()) {
-			for (String key : osm.keySet()) {
-				TreeSet<String> values = null;
-				if (allData.containsKey(key))
-					values = allData.get(key);
-				else {
-					values = new TreeSet<String>();
-					allData.put(key, values);
-				}
-				values.add(osm.get(key));
-			}
-		}
-		for (int i = 0; i < propertyData.getRowCount(); ++i)
-			allData.remove(propertyData.getValueAt(i, 0));
+		final TreeMap<String, TreeSet<String>> allData = createAutoCompletionInfo(false);
 		final AutoCompleteComboBox keys = new AutoCompleteComboBox();
 		keys.setPossibleItems(allData.keySet());
 		keys.setEditable(true);
@@ -240,23 +293,8 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 		final AutoCompleteComboBox values = new AutoCompleteComboBox();
 		values.setEditable(true);
 		p2.add(values, BorderLayout.CENTER);
-	    
-		// get the combo box' editor component
-		JTextComponent editor = (JTextComponent) values.getEditor().getEditorComponent();
-		// Refresh the values model when focus is gained 
-		editor.addFocusListener(new FocusAdapter() {
-            @Override public void focusGained(FocusEvent e) {
-            	String key = keys.getEditor().getItem().toString();
-				Collection<String> newItems;
-            	if (allData.containsKey(key)) {
-					newItems = allData.get(key);
-				} else {
-					newItems = Collections.emptyList();
-				}
-				values.setPossibleItems(newItems);
-            }
-        });
 
+		addFocusAdapter(-1, allData, keys, values);
 		JOptionPane pane = new JOptionPane(p, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION){
 			@Override public void selectInitialValue() {
 				keys.requestFocusInWindow();
@@ -275,6 +313,50 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 		Main.parent.repaint(); // repaint all - drawing could have been changed
 	}
 
+	/**
+	 * @param allData
+	 * @param keys
+	 * @param values
+	 */
+	private void addFocusAdapter(final int row, final TreeMap<String, TreeSet<String>> allData,final AutoCompleteComboBox keys, final AutoCompleteComboBox values) {
+		// get the combo box' editor component
+		JTextComponent editor = (JTextComponent)values.getEditor()
+		        .getEditorComponent();
+		// Refresh the values model when focus is gained
+		editor.addFocusListener(new FocusAdapter() {
+			@Override public void focusGained(FocusEvent e) {
+				String key = keys.getEditor().getItem().toString();
+				updateListData(key, allData, values);
+				objKey=key;
+			}
+		});
+	}
+	private String objKey;
+	/**
+	 * @return
+	 */
+	private TreeMap<String, TreeSet<String>> createAutoCompletionInfo(
+	        boolean edit) {
+		final TreeMap<String, TreeSet<String>> allData = new TreeMap<String, TreeSet<String>>();
+		for (OsmPrimitive osm : Main.ds.allNonDeletedPrimitives()) {
+			for (String key : osm.keySet()) {
+				TreeSet<String> values = null;
+				if (allData.containsKey(key))
+					values = allData.get(key);
+				else {
+					values = new TreeSet<String>();
+					allData.put(key, values);
+				}
+				values.add(osm.get(key));
+			}
+		}
+		if (!edit) {
+			for (int i = 0; i < propertyData.getRowCount(); ++i)
+				allData.remove(propertyData.getValueAt(i, 0));
+		}
+		return allData;
+	}
+
 	/**
 	 * Delete the keys from the given row.
 	 * @param row	The row, which key gets deleted from the dataset.
@@ -294,7 +376,7 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 			return false;
 		}
 		@Override public Class<?> getColumnClass(int columnIndex) {
-			return columnIndex == 1 ? Relation.class : String.class;
+			return String.class;
 		}
 	};
 
@@ -362,10 +444,22 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 			@Override public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
 				Component c = super.getTableCellRendererComponent(table, value, isSelected, false, row, column);
 				if (c instanceof JLabel) {
-					String str = ((JComboBox) value).getEditor().getItem().toString();
+					String str = null;
+					switch (column) {
+					case 0:
+						str = (String) value;
+					break;
+					case 1:
+						Map<String, Integer> v = (Map<String,Integer>) value;
+						if (v.size()!=1) {
+							str=tr("<different>");
+							c.setFont(c.getFont().deriveFont(Font.ITALIC));
+						} else {
+							str=v.entrySet().iterator().next().getKey();
+						}
+					break;
+					}
 					((JLabel)c).setText(str);
-					if (str.equals(tr("<different>")))
-						c.setFont(c.getFont().deriveFont(Font.ITALIC));
 				}
 				return c;
 			}
@@ -458,24 +552,30 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 		
 		propertyData.setRowCount(0);
 
-		Map<String, Integer> valueCount = new HashMap<String, Integer>();
-		TreeMap<String, Collection<String>> props = new TreeMap<String, Collection<String>>();
+		Map<String, Integer> keyCount = new HashMap<String, Integer>();
+		valueCount.clear();
 		for (OsmPrimitive osm : newSelection) {
 			for (Entry<String, String> e : osm.entrySet()) {
-				Collection<String> value = props.get(e.getKey());
-				if (value == null) {
-					value = new TreeSet<String>();
-					props.put(e.getKey(), value);
+				keyCount.put(e.getKey(), keyCount.containsKey(e.getKey()) ? keyCount.get(e.getKey())+1 : 1);
+				if (valueCount.containsKey(e.getKey())) {
+					Map<String, Integer> v = valueCount.get(e.getKey());
+					v.put(e.getValue(), v.containsKey(e.getValue())? v.get(e.getValue())+1 : 1 );
+				} else {
+					TreeMap<String,Integer> v = new TreeMap<String, Integer>();
+					v.put(e.getValue(), 1);
+					valueCount.put(e.getKey(), v);
 				}
-				value.add(e.getValue());
-				valueCount.put(e.getKey(), valueCount.containsKey(e.getKey()) ? valueCount.get(e.getKey())+1 : 1);
 			}
 		}
-		for (Entry<String, Collection<String>> e : props.entrySet()) {
-            JComboBox value = new JComboBox(e.getValue().toArray());
-            value.setEditable(true);
-            value.getEditor().setItem(e.getValue().size() > 1 || valueCount.get(e.getKey()) != newSelection.size() ? tr("<different>") : e.getValue().iterator().next());
-            propertyData.addRow(new Object[]{e.getKey(), value});
+		for (Entry<String, Map<String, Integer>> e : valueCount.entrySet()) {
+			int count=0;
+			for (Entry<String, Integer> e1: e.getValue().entrySet()) {
+				count+=e1.getValue();
+			}
+			if (count < newSelection.size()) {
+				e.getValue().put("", newSelection.size()-count);
+			}
+			propertyData.addRow(new Object[]{e.getKey(), e.getValue()});
 		}
 		
 		// re-load membership data
@@ -496,7 +596,7 @@ public class PropertiesDialog extends ToggleDialog implements SelectionChangedLi
 							roles.put(r, value);
 						}
 						value.add(m.role);
-						valueCountM.put(r, valueCount.containsKey(r) ? valueCount.get(r)+1 : 1);
+						valueCountM.put(r, keyCount.containsKey(r) ? keyCount.get(r)+1 : 1);
 					}
 				}
 			}
